use chrono::{DateTime, Utc};
use serde::Deserialize;

#[derive(Deserialize)]
pub struct Dump {
	pub channel: Channel,
	pub messages: Vec<Message>,
}

#[derive(Deserialize, Clone)]
pub struct Channel {
	#[serde(with = "string")]
	pub id: u64,
	pub name: String,
}

#[derive(Deserialize)]
pub struct Message {
	#[serde(with = "string")]
	pub id: u64,
	pub timestamp: DateTime<Utc>,
	pub content: String,
	pub author: Author,
	pub attachments: Vec<Attachment>,
}

#[derive(Deserialize)]
pub struct Author {
	#[serde(with = "string")]
	pub id: u64,
	pub name: String,
	pub discriminator: String,
}

#[derive(Deserialize)]
pub struct Attachment {
	pub url: String,
}

// https://github.com/serde-rs/json/issues/329#issuecomment-305608405
mod string {
	use serde::{de, Deserialize, Deserializer};
	use std::{fmt::Display, str::FromStr};

	pub fn deserialize<'de, T, D>(deserializer: D) -> Result<T, D::Error>
	where
		T: FromStr,
		T::Err: Display,
		D: Deserializer<'de>,
	{
		String::deserialize(deserializer)?
			.parse()
			.map_err(de::Error::custom)
	}
}
