use std::{fs::File, io::BufReader};

use indicatif::{MultiProgress, ProgressBar, ProgressStyle};

use crate::{dce::Dump, index::IndexConnection};

pub fn indexer(index: String, files: Vec<String>) -> anyhow::Result<()> {
	let index = IndexConnection::open(&index)?;

	let style_int = ProgressStyle::with_template(
		"{spinner:.green} [{bar:40.blue}] {pos}/{len} | {wide_msg:.green}",
	)
	.unwrap()
	.progress_chars("-> ");

	let style_data = ProgressStyle::with_template(
		"{spinner:.green} [{bar:40.blue}] {bytes}/{total_bytes} | {wide_msg:.green}",
	)
	.unwrap()
	.progress_chars("-> ");

	let mbp = MultiProgress::new();
	let pb_file = mbp.add(ProgressBar::new(files.len() as u64 + 1));
	pb_file.set_style(style_int.clone());

	for file in pb_file.wrap_iter(files.into_iter()) {
		pb_file.set_message(file.clone());
		let file = File::open(file)?;

		let pb_serialize =
			mbp.add(ProgressBar::new(file.metadata()?.len()).with_message("Deserializing..."));
		pb_serialize.set_style(style_data.clone());

		let dump: Dump = serde_json::from_reader(BufReader::new(pb_serialize.wrap_read(file)))?;
		let channel_id = dump.channel.id;
		index.add_channel(dump.channel)?;

		let pb_index =
			mbp.add(ProgressBar::new(dump.messages.len() as u64).with_message("Indexing..."));
		pb_index.set_style(style_int.clone());

		for msg in pb_index.wrap_iter(dump.messages.into_iter()) {
			index.add_msg(msg, channel_id)?;
		}

		mbp.remove(&pb_index);
		mbp.remove(&pb_serialize);
	}
	eprintln!("Optimizing the index...");
	index.optimize()?;
	eprintln!("Done!");
	Ok(())
}
