use clap::Parser;
use mimalloc::MiMalloc;

#[global_allocator]
static GLOBAL: MiMalloc = MiMalloc;

mod browse;
mod config;
mod dce;
mod index;
mod indexer;
mod strchunks;

#[derive(Parser, Debug)]
#[clap(author, version, about, long_about = None)]
pub enum Cli {
	/// Create an index from DCE dumps
	Index {
		/// Output index filename
		index: String,
		/// DCE dumps to index
		files: Vec<String>,
	},
	/// Browse an index with a TUI
	Browse {
		/// Location of the index
		index: String,
	},
}

fn main() {
	let result = match Cli::parse() {
		Cli::Index { index, files } => indexer::indexer(index, files),
		Cli::Browse { index } => browse::browse(index),
	};
	if let Err(e) = result {
		eprintln!("An error has occurred: {e}");
	}
}
