pub struct StrChunks<'a> {
	v: &'a str,
	size: usize,
}

impl<'a> Iterator for StrChunks<'a> {
	type Item = &'a str;
	fn next(&mut self) -> Option<Self::Item> {
		if self.v.is_empty() {
			return None;
		}
		if self.v.len() < self.size {
			let res = self.v;
			self.v = &self.v[self.v.len()..];
			return Some(res);
		}
		let mut offset = self.size;
		let res = loop {
			match self.v.get(..offset) {
				Some(v) => break v,
				None => {
					offset -= 1;
				}
			}
		};
		self.v = &self.v[offset..];
		Some(res)
	}
}

impl<'a> StrChunks<'a> {
	pub fn new(v: &'a str, size: usize) -> Self {
		Self { v, size }
	}
}
